%%the cell_data structure can be downloaded from the publication at https://www.embopress.org/doi/full/10.15252/msb.20178140
%%all the data is contained in what is named cell_data in the following scripts
clear all
load cell_data

%%Figure 1(a) 
%OCT4 over time for both pluripotent and differentiate cell groups

figure
countpp=0; %count of pluripotent cells
countdiff=0; %count of differentiated cells
for i=1:1274 %for every cell
    y=[cell_data(i).Fate]; %cell fate
    cellt=[cell_data(i).t]; %cell time
    oct=[cell_data(i).MeanIntensity]; %cell OCT4
    cellt=cellt(~isnan(cellt)); %remove NaN from time
    oct=oct(~isnan(oct)); %remove NaN from OCT4

    if strcmp(y,'Pluripotent')==1 %if cell is pluripotent
        countpp=countpp+1; %increase pp cell counter
        ppid(countpp)=i; %save pp cell ID
        plot(cellt/12,oct,'Color',[0.3 0.3 0.8]) %plot every time series in purple
        hold on
    end 
end
for i=1:1274 %note: done as two loops to plot differentiated cells over the top for better visual
    y=[cell_data(i).Fate];
    cellt=[cell_data(i).t]; %cell time
    oct=[cell_data(i).MeanIntensity]; %cell OCT4
    cellt=cellt(~isnan(cellt)); %remove NaN from time
    oct=oct(~isnan(oct)); %remove NaN from OCT4

    if strcmp(y,'Differentiated')==1 %if cell is differentiated
        countdiff=countdiff+1; %increase diff cell counter
        diffid(countdiff)=i; %save diff cell ID
        plot(cellt/12,oct,'Color',[0.1 0.8 0.2]) %plot every time series in green
        hold on
    end 
 end 
xlabel('Time (hours)')
ylabel('OCT4')
xlim([0 68])
xticks([0:20:60])
ax = gca;
ax.YAxis.Exponent = 3;
ylim([0 3150])
plot([43 43],[0 3150],'--k') %dashed line at BMP4 addition





%%Figure 1(b)
%distribution of OCT4 values pre-differentiation
OCTsavepp=[];
OCTsave=[];
OCTsavediff=[];
for i=1:1274
        cellt=[cell_data(i).t]; %cell time
        oct=[cell_data(i).MeanIntensity]; %OCT4
        if length(cellt(cellt>516))~=0 % if cell lives past 40 hours 
         f=find(cellt>516); %find the OCT values   
         oct(f)=[]; %and remove them
        end
        oct=oct(~isnan(oct)); %remove NaN from OCT4
        if ismember(i,ppid) %if cell is pluripotent (from code for fig 1a) 
         OCTsavepp(i,1:length(oct))=oct; %save pp OCT values
        elseif ismember(i,diffid) %if cell is differentiated (from code for fig 1a)
         OCTsavediff(i,1:length(oct))=oct; %save diff OCT values
        end
        
        OCTsave(i,1:length(oct))=oct; %save all OCT values
end

%collect all OCT4 values as long vector for each for histogram
%remove the zeros 
octallpp=OCTsavepp(:);
octallpp=octallpp(octallpp~=0);
octalldiff=OCTsavediff(:);
octalldiff=octalldiff(octalldiff~=0);
octall=OCTsave(:);
octall=octall(octall~=0);
bins=[-0:150:3000]; %bin edges

%histogram pluripotent
h1=histogram(octallpp,bins,'Normalization','pdf')
y1=h1.Values; %get bin heights
b1=h1.BinEdges(2:end)+h1.BinWidth/2; %get bin centres

%histogram differentiated
h2=histogram(octalldiff,bins,'Normalization','pdf')
y2=h2.Values; %get bin heights
b2=h2.BinEdges(2:end)+h2.BinWidth/2; %get bin centres

%histogram all
h3=histogram(octall,bins,'Normalization','pdf')
y3=h3.Values; %get bin heights
b3=h3.BinEdges(2:end)+h3.BinWidth/2; %get bin centres

%plot figure with bin centres and heights
figure
plot(b3,y3,'-o','Color',[0.9 0.4 0],'MarkerSize',3, 'Markerfacecolor',[0.9 0.4 0])
hold on
plot(b1,y1,'-s','Color',[0.3 0.3 0.8],'MarkerSize',4, 'Markerfacecolor',[0.3 0.3 0.8])
plot(b2,y2,'-d','Color',[0.1 0.8 0.2],'MarkerSize',3,'Markerfacecolor',[1 1 1])
xlabel('OCT4')
ylabel('Probability density')
xlim([0 3000])
ylim([0 0.0018])
ax = gca;
ax.YAxis.Exponent = -3;





%%Figure 1(c)
%temporal distribtuions for pluripotent cells

groupspp=10; %number of groups to split into
countpp=ones(1,groupspp); %set up counter for each group
groupdistpp=40/groupspp; %bin width in hours
tpp(1)=0;
tpp(2:groupspp+1)=[1:groupspp]*groupdistpp*12; %bin edges
groupcountspp=zeros(1,groupspp); %empty group counts
%find cells in each category
for i=ppid %for every pluripotent cell
    cellt=[cell_data(i).t]; %cell time
    oct=[cell_data(i).MeanIntensity]; %cell OCT4
    cellt=cellt(~isnan(cellt)); 
    oct=oct(~isnan(oct));
    
    %want to bin OCT values by t
    Ypp = discretize(cellt,tpp);

    %collect values in each group
    for j=1:groupspp
       data=oct(Ypp==j,:);
       octgroupspp(j,groupcountspp(j)+1:groupcountspp(j)+length(data)) = data;
       groupcountspp(j)=groupcountspp(j)+length(data);
    end 
   
end

%fit distribtuion for each group
xvals=[0:0.1:3000];
for k=1:groupspp
    octdata=octgroupspp(k, :);
    octdata=octdata(octdata~=0);
    Noctpp(k)=length(octdata);
    pdf1=fitdist(octdata','Kernel','Bandwidth',50);
    yvalspp(k,1:length(xvals))=pdf(pdf1,xvals);
end 

%%ks test for pluripotent cells
for i=1:9
kstest2(octgroupspp(i,1:Noctpp(i)),octgroupspp(i+1,1:Noctpp(i+1)));
end 

for i=1:10
    xvals2(i,:)=xvals;
    groupid(i,:)=tpp(i).*ones(length(xvals),1)./12;
end 

figure
scatter(xvals2(:),yvalspp(:),1,groupid(:))
xlabel('OCT4')
ylabel('Probability density')
xlim([250 3000])
c = colorbar('east');
c.Label.String = 'Time (hours)';
c.FontSize = 8;
box on 




%%Figure 1(d)
%temporal distributions for differentiated cells

groupsdiff=10; 
countdiff=ones(1,groupsdiff);
groupdistdiff=40/groupsdiff;
tdiff(1)=0;
tdiff(2:groupsdiff+1)=[1:groupsdiff]*groupdistdiff*12;
groupcountsdiff=zeros(1,groupsdiff);
%find cells in each category
for i=diffid
    cellt=[cell_data(i).t];
    oct=[cell_data(i).MeanIntensity];
    cellt=cellt(~isnan(cellt));
    oct=oct(~isnan(oct));
    
    %want to bin OCT values by t
    Ydiff = discretize(cellt,tdiff);

    
    %collect values in each group
    for j=1:groupsdiff
       data=oct(Ydiff==j,:);
       octgroupsdiff(j,groupcountsdiff(j)+1:groupcountsdiff(j)+length(data)) = data;
       groupcountsdiff(j)=groupcountsdiff(j)+length(data);
    end 
   
end

for k=1:groupsdiff
    octdata=octgroupsdiff(k, :);
    octdata=octdata(octdata~=0);
    Noctdiff(k)=length(octdata);
    pdf12=fitdist(octdata','Kernel','Bandwidth',50);
    yvalsdiff(k,1:length(xvals))=pdf(pdf12,xvals);
end

%%ks test for diff
for i=1:9
kstest2(octgroupsdiff(i,1:Noctdiff(i)),octgroupsdiff(i+1,1:Noctdiff(i+1)));
end 

figure
scatter(xvals2(:),yvalsdiff(:),1,groupid(:))
xlabel('OCT4')
ylabel('Probability density')
xlim([250 3000])
c = colorbar('east');
c.Label.String = 'Time (hours)';
c.FontSize = 8;
box on 
