%%the cell_data structure can be downloaded from the publication at https://www.embopress.org/doi/full/10.15252/msb.20178140
%%all the data is contained in what is named cell_data in the following scripts


%%Figure 3(a)
%Simulated OCT4 expression using SLE with additive noise

load cell_data %%data set 
load cckernel %%use cckernel from initial conditions script or other cell cycle kernel of your choice
load initialoctkernel %%use initialoct kernel from initial conditions script of kernel of your choice

ccpd=cckernel;

N=16; %starting number of cells
events=300; %how many divisions do we want to occur
OCT4=[]; %set up empty OCT4 matrix
r=0.02; %regulation parameter r
H=0.38; %Hurst exponent
k=1290; %carrying capacity
sigma=90; %additive noise 
dt=1; %timestep

%set up initial cells
for i=1:N %for every initial cell
cctime(i)=random(cckernel)*12; %initial cell cycle timeframes
cellstartingtime(i)= unifrnd(0,cctime(i)); %choose a random time between 0 and the cell cycle time for the cell to begin at 
cctimeframes(i)=cctime(i)-cellstartingtime(i); %remainder of cell cycle
initialoct(i)=random(initialoctkernel); %initial OCT4 values
OCT4(1:floor(cctimeframes(i)),i)=logisticsdefbm(r, k, sigma, initialoct(i), 1, floor(cctimeframes(i)), dt,H); %%first cells OCT4 time series
celldivisiontimes(i)=floor(cctimeframes(i)); %cell division time
if celldivisiontimes(i)==0 %correct for if the division time is 0 and make it 1 
    celldivisiontimes(i)=1;
end
end

for j=1:events %for every division event
dividingcellID=find(celldivisiontimes==min(celldivisiontimes)); %find cell dividing next
if length(dividingcellID)>1 %if two cells want to divide at once then pick one randomly
pos = randi(length(dividingcellID));
cellid=dividingcellID(pos);
else 
cellid=dividingcellID;
end
newcctime1=random(ccpd)*12;
newcctime2=random(ccpd)*12;
while newcctime1<100 %correct for long cell cycle tails
    newcctime1=random(ccpd)*12;
end 
while newcctime2<100
    newcctime2=random(ccpd)*12;
end 

motherfinaloct=OCT4(celldivisiontimes(cellid),cellid); %note the final OCT4 value of mother cell
OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime1),cellid)= logisticsdefbm(r, k, sigma, motherfinaloct, 1, floor(newcctime1), dt, H); %update current cell OCT4 time series
OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime2),N+1)=logisticsdefbm(r, k, sigma, motherfinaloct, 1, floor(newcctime2), dt, H); %new cell OCT4 time series
celldivisiontimes(N+1)=celldivisiontimes(cellid)+floor(newcctime2); %new cell division time
celldivisiontimes(cellid)= celldivisiontimes(cellid)+floor(newcctime1); %update cell division time for the cell that's dividing
N=N+1; %number of cells counter
end


timesteps=[1:1:length(OCT4)];

%plotting
figure
for i=1:N %for every cell
    octvals=OCT4(:,i);
    f=find((octvals~=0));
    plot(timesteps(f)/12,octvals(f),'Color',[0.9 0.4 0 0.4])
    lifetimes=length(f);
    hold on 
end
%optional plotting for data over the top
countpp=0; %count of pluripotent cells
countdiff=0; %count of differentiated cells
for i=1:1274 %for every cell
    y=[cell_data(i).Fate]; %cell fate
    cellt=[cell_data(i).t]; %cell time
    oct=[cell_data(i).MeanIntensity]; %cell OCT4
    cellt=cellt(~isnan(cellt)); %remove NaN from time
    oct=oct(~isnan(oct)); %remove NaN from OCT4

    if strcmp(y,'Pluripotent')==1 %if cell is pluripotent
        countpp=countpp+1; %increase pp cell counter
        ppid(countpp)=i; %save pp cell ID
        plot(cellt/12,oct,'Color',[0.3 0.3 0.8]) %plot every time series in purple
        hold on
    end 
end
for i=1:1274 %note: done as two loops to plot differentiated cells over the top for better visual
    y=[cell_data(i).Fate];
    cellt=[cell_data(i).t]; %cell time
    oct=[cell_data(i).MeanIntensity]; %cell OCT4
    cellt=cellt(~isnan(cellt)); %remove NaN from time
    oct=oct(~isnan(oct)); %remove NaN from OCT4

    if strcmp(y,'Differentiated')==1 %if cell is differentiated
        countdiff=countdiff+1; %increase diff cell counter
        diffid(countdiff)=i; %save diff cell ID
        plot(cellt/12,oct,'Color',[0.1 0.8 0.2]) %plot every time series in green
        hold on
    end 
 end 
xlabel('Time (hours)')
ylabel('OCT4')
ylim([0 3150])
ax = gca;
ax.YAxis.Exponent = 3;
xlim([0 40])



%%Figure 3(b)
%corresponding histogram

%%plot histogram of all OCT4 values 
OCT40to40=OCT4(1:480,:);
OCT4all=OCT4(:);
OCT4all=OCT4all(OCT4all~=0);

load octdistkernel %distribution kernel from the data

bins=[-0:150:3000];
figure
histogram(OCT4all,bins,'Normalization','pdf','FaceColor', [0 0.45 0.74],'LineWidth',0.8,'FaceAlpha', 1)
xlabel('OCT4')
ylabel('Probability density')
xlim([-500 3000])
hold on
plot([1290 1290],[0, 0.0015],'Linewidth', 1.5, 'Color', [0.9 0.4 0]) %line for carrying capacity
xvals=[-2000:4000];
yvals=pdf(pd1,xvals);
plot(xvals,yvals,'LineWidth',1.5,'Color', [0.9 0.4 0]) %data distribution 
ylim([0 0.0014])
