%%the cell_data structure can be downloaded from the publication at https://www.embopress.org/doi/full/10.15252/msb.20178140
%%all the data is contained in what is named cell_data in the following scripts

clear all

%%Figure 4(a)
%Simulated OCT4 expression using SLE with a noise switch

load cell_data %%data set 
load cckernel %%use cckernel from initial conditions script or other cell cycle kernel of your choice
load initialoctkernel %%use initialoct kernel from initial conditions script of kernel of your choice

pd2=makedist('Uniform',900,1200); %a distribution for the initial conditions of the differentiated cells
ccpd=cckernel;

Npp=14; %starting number of pluripotent cells
Ndiff=2;  %starting number of differentiated cells
eventspp=500; %how many divisions for pp cells
eventsdiff=200; %how many divisions for diff cells
OCT4=zeros(10,100); %set up empty OCT4 matrix
r=0.01; %regulation parameter r
kpp=1290; %carrying capacity pp cells
kdiff=900; %carrying capacity diff cells
sigmaadiff=90; %additive noise diff cells
sigmaapp=90; %additive noise pp cells
sigmampp=0; %multiplicative noise pp cells
sigmamdiff=0; %multiplicative noise diff cells
dt=1; %time step
switchtime=20; %time to turn on multiplicative noise in hours
H=0.38; %Hurst exponent

%set up pp cells
for i=1:Npp 
cctime(i)=random(ccpd)*12; %cell initial cell cycle timeframes
cellstartingtime(i)= unifrnd(0,cctime(i));%choose a random time between 0 and the cell cycle time for the cell to begin at 
cctimeframes(i)=cctime(i)-cellstartingtime(i); %remaining cell cycle time
initialoct(i)=log(random(initialoctkernel)); %initial OCT value
OCT4pp(1:floor(cctimeframes(i)),i)=loglogistic_bothnoise(initialoct(i), 1, floor(cctimeframes(i)), dt, kpp, r, sigmaapp, sigmampp,H); %%first cells OCT4 time series
celldivisiontimespp(i)=floor(cctimeframes(i)); %cell division times
if celldivisiontimespp(i)==0 %correct for if the division time is 0 and make it 1 
    celldivisiontimespp(i)=1;
end 
end 

%set up diff cells
for i=1:Ndiff
cctime(i)=random(ccpd)*12; %cell initial cell cycle timeframes
cellstartingtime(i)= unifrnd(0,cctime(i));%choose a random time between 0 and the cell cycle time for the cell to begin at 
cctimeframes(i)=cctime(i)-cellstartingtime(i); %remaining cell cycle time
initialoct(i)=log(random(pd2)); %initial OCT4 value
OCT4diff(1:floor(cctimeframes(i)),i)=loglogistic_bothnoise(initialoct(i), 1, floor(cctimeframes(i)), dt, kdiff, r, sigmaadiff, sigmamdiff,H); %%first cells OCT4 values
celldivisiontimesdiff(i)=floor(cctimeframes(i)); %cell division times
if celldivisiontimesdiff(i)==0 %correct for if the division time is 0 and make it 1 
    celldivisiontimesdiff(i)=1;
end 
end

%events for pp cells
for j=1:eventspp
dividingcellID=find(celldivisiontimespp==min(celldivisiontimespp)); %find next cell dividing
if length(dividingcellID)>1 %if two cells want to divide at once pick one randomly
pos = randi(length(dividingcellID));
cellid=dividingcellID(pos);
else 
cellid=dividingcellID;
end
newcctime1=random(ccpd)*12;
newcctime2=random(ccpd)*12;
while newcctime1<100
    newcctime1=random(ccpd)*12;
end 
while newcctime2<100
    newcctime2=random(ccpd)*12;
end 

%if time is past 20 hours
if celldivisiontimespp(cellid)>switchtime*12
    sigmaapp=0; %replace sigma with 0 
    sigmampp=0.055; %replace sigmam with 0.025
end 

motherfinaloct=OCT4pp(celldivisiontimespp(cellid),cellid); %note final mother cell OCT4
OCT4pp(celldivisiontimespp(cellid)+1:celldivisiontimespp(cellid)+floor(newcctime1),cellid)= loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime1), dt, kpp, r, sigmaapp,sigmampp,H); %update current cell
OCT4pp(celldivisiontimespp(cellid)+1:celldivisiontimespp(cellid)+floor(newcctime2),Npp+1)=loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime2), dt, kpp, r, sigmaapp,sigmampp,H); %new cell
celldivisiontimespp(Npp+1)=celldivisiontimespp(cellid)+floor(newcctime2); %new cell division time
celldivisiontimespp(cellid)= celldivisiontimespp(cellid)+floor(newcctime1); %update cell division time for the cell that's dividing
Npp=Npp+1; %number of cells counter
end

%%events for diff cells
for j=1:eventsdiff
dividingcellID=find(celldivisiontimesdiff==min(celldivisiontimesdiff)); %find next cell dividing
if length(dividingcellID)>1 %if two cells want to divide at once pick one randomly
pos = randi(length(dividingcellID));
cellid=dividingcellID(pos);
else 
cellid=dividingcellID;
end
newcctime1=random(ccpd)*12;
newcctime2=random(ccpd)*12;
while newcctime1<100
    newcctime1=random(ccpd)*12;
end 
while newcctime2<100
    newcctime2=random(ccpd)*12;
end 

motherfinaloct=OCT4diff(celldivisiontimesdiff(cellid),cellid); %note mother cell final OCT
OCT4diff(celldivisiontimesdiff(cellid)+1:celldivisiontimesdiff(cellid)+floor(newcctime1),cellid)= loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime1), dt, kdiff, r, sigmaadiff,sigmamdiff,H); %update current cell
OCT4diff(celldivisiontimesdiff(cellid)+1:celldivisiontimesdiff(cellid)+floor(newcctime2),Ndiff+1)=loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime2), dt, kdiff, r, sigmaadiff,sigmamdiff,H); %new cell
celldivisiontimesdiff(Ndiff+1)=celldivisiontimesdiff(cellid)+floor(newcctime2); %new cell division time
celldivisiontimesdiff(cellid)= celldivisiontimesdiff(cellid)+floor(newcctime1); %update cell division time for the cell that's dividing
Ndiff=Ndiff+1; %number of cells counter
end

%plot the OCT4 over time
timestepspp=[1:1:length(OCT4pp)];
timestepsdiff=[1:1:length(OCT4diff)];
Time=68;
%%plotting
figure
for i=1:Npp %pluripotent cells
    octvals=(OCT4pp(:,i));
    f=find((octvals>0) & (octvals<10));
    plot(timestepspp(f)/12,exp(octvals(f)),'Color',[0.3 0.3 0.8])
    lifetimes=length(f);
    hold on 
end
for i=1:Ndiff %differentiated cells
    octvals=(OCT4diff(:,i));
    f=find((octvals>0) & (octvals<10));
    plot(timestepsdiff(f)/12,exp(octvals(f)),'Color',[0.1 0.8 0.2])
    lifetimes=length(f);
    hold on 
end
xlabel('Time (hours)')
ylabel('OCT4')
xlim([0 40])
ax = gca;
ax.YAxis.Exponent = 3;
xlim([0 40])
ylim([0 3150])






%%Figure 4(b)
%corresponding histogram

%%plot histogram of all OCT4 values 
OCT40to40pp=OCT4pp(1:480,:);
OCT4allpp=OCT40to40pp(:);
OCT4allpp=OCT4allpp(OCT4allpp~=0);
OCT40to40diff=OCT4diff(1:480,:);
OCT4alldiff=OCT40to40diff(:);
OCT4alldiff=OCT4alldiff(OCT4alldiff~=0);
OCTall=[OCT4alldiff; OCT4allpp]';


bins=[-0:150:3000];
figure
histogram(exp(OCT4allpp),bins,'Normalization','pdf','FaceColor', [0.3 0.3 0.8],'LineWidth',0.8,'FaceAlpha', 0.5)
xlabel('OCT4')
ylabel('Probability density')
xlim([0 3000])
hold on
histogram(exp(OCT4alldiff),bins,'Normalization','pdf','FaceColor', [0.1 0.8 0.2],'LineWidth',0.8,'FaceAlpha', 0.5)
ylim([0 0.0018])





%%Figure 4(c)
%split by time intervals for pluripotent cells 

groups=8; %number of groups to split into
groupspp=[1:(40*12-1)/groups:40*12];
groupspp=round(groupspp);
xvals=[0:0.1:3000];
for i=1:length(groupspp)-1
    selectoctpp=OCT4pp(groupspp(i):groupspp(i+1),:);
    selectoctpp=selectoctpp(selectoctpp~=0);
    expvalspp=exp(selectoctpp(:));
    expvalspp=expvalspp(expvalspp~=inf);
    pdfpp=fitdist(expvalspp,'Kernel','Bandwidth',100);
    yvalskernelpp(i,1:length(xvals))=pdf(pdfpp,xvals); 
end

for i=1:groups
    xvalspp2(i,:)=xvals;
    groupidpp(i,:)=groupspp(i).*ones(length(xvals),1)./12;
end 

figure
scatter(xvalspp2(:),yvalskernelpp(:),1,groupidpp(:))
xlabel('OCT4')
ylabel('Probability density')
xlim([0 3000])
c = colorbar('east');
c.Label.String = 'Time (hours)';
c.FontSize = 8;
box on 
ylim([0 0.002])






%%Figure 4(d)
%split by time intervals for differentiated cells 

groupsdiff=[1:(40*12-1)/groups:40*12];
groupsdiff=round(groupsdiff);
xvals=[0:0.1:3000];
for i=1:length(groupsdiff)-1
    selectoctdiff=OCT4diff(groupsdiff(i):groupsdiff(i+1),:);
    selectoctdiff=selectoctdiff(selectoctdiff~=0);
    expvalsdiff=exp(selectoctdiff(:));
    expvalsdiff=expvalsdiff(expvalsdiff~=inf);
    pdfdiff=fitdist(expvalsdiff,'Kernel','Bandwidth',100);
    yvalskerneldiff(i,1:length(xvals))=pdf(pdfdiff,xvals); 
end

for i=1:groups
    xvalsdiff2(i,:)=xvals;
    groupiddiff(i,:)=groupsdiff(i).*ones(length(xvals),1)./12;
end 

figure
scatter(xvalsdiff2(:),yvalskerneldiff(:),1,groupiddiff(:))
xlabel('OCT4')
ylabel('Probability density')
xlim([0 3000])
c = colorbar('east');
c.Label.String = 'Time (hours)';
c.FontSize = 8;
box on 
ylim([0 0.002])

