function [Xt] = eulersdefbm( x0, t0, tf, dt, f, g, H)
% x0 -- initial value of random variable
% t0 -- start time
% tf -- end time
% dt -- time increment
% f -- function associated with the deterministic part of the SDE
% g -- funciton associated with the stochastic part of the SDE
t=t0:dt:tf; %sets time interval
n=max(size(t)); %determines number of iterations
sqrtdt=sqrt(dt);
Xt=zeros(n,1); %initializes vector and sets the initial condition
Xt(1)=x0;
fbmnoise=ffgn(1,H,1,n,0); %generate the fractional brownian noise

for i=1:n-1
Xt(i+1)=Xt(i)+f(Xt(i))*dt+g(Xt(i))*fbmnoise(i)*sqrtdt; 
end
end
